@@ -25,12 +25,12 @@ from vellum.workflows.nodes.utils import get_unadorned_node, get_unadorned_port,
 from vellum.workflows.ports import Port
 from vellum.workflows.references import OutputReference, WorkflowInputReference
 from vellum.workflows.state.encoder import DefaultStateEncoder
+from vellum.workflows.triggers.integration import IntegrationTrigger
 from vellum.workflows.triggers.manual import ManualTrigger
-from vellum.workflows.triggers.vellum_integration import VellumIntegrationTrigger
 from vellum.workflows.types.core import Json, JsonArray, JsonObject
 from vellum.workflows.types.generics import WorkflowType
 from vellum.workflows.types.utils import get_original_base
-from vellum.workflows.utils.uuids import get_trigger_id, uuid4_from_hash
+from vellum.workflows.utils.uuids import uuid4_from_hash
 from vellum.workflows.utils.vellum_variables import primitive_type_to_vellum_variable_type
 from vellum.workflows.vellum_client import create_vellum_client
 from vellum_ee.workflows.display.base import (
@@ -190,7 +190,7 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
         # For IntegrationTrigger only: no ENTRYPOINT node (use trigger ID directly in edges)
         manual_trigger_edges = [edge for edge in trigger_edges if issubclass(edge.trigger_class, ManualTrigger)]
         integration_trigger_edges = [
-            edge for edge in trigger_edges if issubclass(edge.trigger_class, VellumIntegrationTrigger)
+            edge for edge in trigger_edges if issubclass(edge.trigger_class, IntegrationTrigger)
         ]
 
         has_manual_trigger = len(manual_trigger_edges) > 0
@@ -202,7 +202,7 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
         if has_manual_trigger:
             # ManualTrigger: use trigger ID for ENTRYPOINT node (backward compatibility)
             trigger_class = manual_trigger_edges[0].trigger_class
-            entrypoint_node_id = get_trigger_id(trigger_class)
+            entrypoint_node_id = trigger_class.__id__
             entrypoint_node_source_handle_id = self.display_context.workflow_display.entrypoint_node_source_handle_id
 
             # Add ENTRYPOINT node for ManualTrigger workflows
@@ -386,7 +386,7 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
         if has_only_integration_trigger:
             # Use trigger ID directly as sourceNodeId (no ENTRYPOINT node)
             trigger_class = integration_trigger_edges[0].trigger_class
-            trigger_id = get_trigger_id(trigger_class)
+            trigger_id = trigger_class.__id__
             trigger_source_handle_id = trigger_id  # Use trigger ID as handle ID
 
             for target_node, entrypoint_display in self.display_context.entrypoint_displays.items():
@@ -536,12 +536,12 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
                     f"{edge.trigger_class.__name__} in the same workflow."
                 )
 
-        # Get the trigger type from the mapping, or check if it's a VellumIntegrationTrigger subclass
+        # Get the trigger type from the mapping, or check if it's an IntegrationTrigger subclass
         trigger_type = trigger_type_mapping.get(trigger_class)
         if trigger_type is None:
-            # Check if it's a VellumIntegrationTrigger subclass
+            # Check if it's an IntegrationTrigger subclass
 
-            if issubclass(trigger_class, VellumIntegrationTrigger):
+            if issubclass(trigger_class, IntegrationTrigger):
                 trigger_type = WorkflowTriggerType.INTEGRATION
             else:
                 raise ValueError(
@@ -550,7 +550,7 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
                 )
 
         # Return as a list with a single trigger object matching Django schema
-        trigger_id = get_trigger_id(trigger_class)
+        trigger_id = trigger_class.__id__
 
         # Serialize trigger attributes like node outputs
         attribute_references = trigger_class.attribute_references().values()
