@@ -1,4 +1,4 @@
-from typing import Any, Dict
+from typing import Any, Dict, Union
 
 from pydantic import Field, field_serializer
 
@@ -12,25 +12,27 @@ class DatasetRow(UniversalBaseModel):
 
     Attributes:
         label: String label for the dataset row
-        inputs: BaseInputs instance containing the input data
+        inputs: BaseInputs instance or dict containing the input data
     """
 
     label: str
-    inputs: BaseInputs = Field(default_factory=BaseInputs)
+    inputs: Union[BaseInputs, Dict[str, Any]] = Field(default_factory=BaseInputs)
 
     @field_serializer("inputs")
-    def serialize_inputs(self, inputs: BaseInputs) -> Dict[str, Any]:
+    def serialize_inputs(self, inputs: Union[BaseInputs, Dict[str, Any]]) -> Dict[str, Any]:
         """
-        Custom serializer for BaseInputs that converts it to a dictionary.
+        Custom serializer for inputs that converts it to a dictionary.
 
         Args:
-            inputs: BaseInputs instance to serialize
+            inputs: Either a BaseInputs instance or dict to serialize
 
         Returns:
             Dictionary representation of the inputs
         """
-        result = {}
+        if isinstance(inputs, dict):
+            return inputs
 
+        result = {}
         for input_descriptor, value in inputs:
             if not input_descriptor.name.startswith("__"):
                 result[input_descriptor.name] = value
